<?php

class WGMT_Merchant {
	/**
	 * Get products according to feed config.
	 *
	 * @param $feed
	 *
	 * @return array|stdClass
	 */
	protected function wgmt_feeds_get_filtered_products( $feed ) {
		$args              = array( 'limit' => - 1 );
		$products_type     = array( 'simple' );
		$products_category = array();

		/** Type of products */
		$include_grouped = array_key_exists( 'feeds_config_grouped', $feed['config'] ) && $feed['config']['feeds_config_grouped'] == 'on' ? true : false;
		if ( $include_grouped ) {
			array_push( $products_type, 'grouped' );
		}

		$include_variable = array_key_exists( 'feeds_config_variable', $feed['config'] ) && $feed['config']['feeds_config_variable'] == 'on' ? true : false;
		if ( $include_variable ) {
			array_push( $products_type, 'variable' );
		}

		$include_external = array_key_exists( 'feeds_config_external', $feed['config'] ) && $feed['config']['feeds_config_external'] == 'on' ? true : false;
		if ( $include_external ) {
			array_push( $products_type, 'external' );
		}

		/** Categories */
		$categories        = array_key_exists( 'feeds_config_categories', $feed['config'] ) ? $feed['config']['feeds_config_categories'] : array();
		$products_category = array_merge( $products_category, $categories );

		/** Fill arguments */
		$args['type']     = $products_type;
		$args['category'] = $products_category;

		return wc_get_products( $args );
	}

	/**
	 * Get the raw value of the item.
	 *
	 * @param $tag
	 * @param WC_Product $product
	 *
	 * @return mixed|string
	 */
	protected function wgmt_feeds_get_item_raw_value( $tag, WC_Product $product ) {
		$value = "";

		if ( array_key_exists( 'value', $tag ) ) {
			if ( $tag['value'] == 'string_static' && array_key_exists( 'static', $tag ) ) {
				$value = $tag['static'];
			} else {
				$value = $this->wgmt_feeds_get_value_from_exp( $tag['value'], $product );
			}
		}

		return $value;
	}

	/**
	 * Validate the condition of the tag.
	 *
	 * @param WC_Product $product
	 * @param $tag
	 * @param $item_value
	 * @param $main_condition
	 *
	 * @return bool
	 */
	protected function wgmt_feeds_validate_tag( WC_Product $product, $tag, $item_value, $main_condition ) {
		$test = true;
		if ( ( array_key_exists( 'if', $tag ) && ! empty( $tag['if'] ) ) && ( array_key_exists( 'condition', $tag ) && ! empty( $tag['condition'] ) ) ) {
			switch ( $tag['if'] ) {
				case '=':
					$test = $item_value == $tag['condition'];
					break;
				case '!=':
					$test = $item_value != $tag['condition'];
					break;
				case '>':
					$test = $item_value > $tag['condition'];
					break;
				case '<':
					$test = $item_value < $tag['condition'];
					break;
				case '>=':
					$test = $item_value >= $tag['condition'];
					break;
				case '<=':
					$test = $item_value <= $tag['condition'];
					break;
				case 'in':
					$test = stristr( $item_value, $tag['condition'] ) !== false;
					break;
				case 'ex':
					$test = stristr( $item_value, $tag['condition'] ) === false;
					break;
				default:
					$test = false;
					break;
			}
		}

		return $test;
	}

	/**
	 * Format the item value according to preset.
	 *
	 * @param $tag
	 * @param $value
	 * @param array $field
	 *
	 * @return string
	 */
	protected function wgmt_feeds_get_formatted_value( $tag, $value, $field = array() ) {
		/** Merchant fields */
		if ( ! empty( $field ) ) {
			/** Apply format */
			if ( ! empty( $field['format'] ) ) {
				$value = $this->wgmt_feeds_format_value( $value, $field['format'] );
			}

			/** Apply suffix */
			if ( ! empty( $field['dynamic_suffix'] ) ) {
				$value .= ' ' . $field['dynamic_suffix'];
			}
		}

		/** CDATA */
		if ( array_key_exists( 'cdata', $tag ) && $tag['cdata'] == 'on' ) {
			$value = "<![CDATA[{$value}]]>";
		}

		return $value;
	}

	/**
	 * Get the tag value from expression.
	 *
	 * @param $exp
	 * @param WC_Product $product
	 *
	 * @return mixed
	 */
	private function wgmt_feeds_get_value_from_exp( $exp, WC_Product $product ) {
		/** Determine function to get value */
		$exp_parts      = explode( '_', $exp );
		$exp_identifier = $exp_parts[0];
		unset( $exp_parts[0] );
		$exp_term = implode( '_', $exp_parts );

		/** Possible functions args */
		$args = array(
			'product' => $product,
		);

		/** Select the proper function to get tag value */
		switch ( $exp_identifier ) {
			case 'attr':
				$value = $product->get_attribute( $exp_term );
				break;
			case 'term':
				/** @var WP_Term $term */
				if ( $product->get_type() == 'variation' ) {
					$term = wp_get_post_terms( $product->get_parent_id(), $exp_term )[0];
				} else {
					$term = wp_get_post_terms( $product->get_id(), $exp_term )[0];
				}
				$value = $term->name;
				break;
			case 'self':
				$func = 'wgmt_feeds_get_self_value_' . $exp_term;
				if ( method_exists( $this, $func ) ) {
					$value = $this->$func( $args );
				} else {
					$value = "";
				}
				break;
			case 'meta':
				$value = get_post_meta( $product->get_id(), $exp_term )[0];
				break;
			case 'taxonomy':
				/** @var WP_Term $category */
				$category = wp_get_post_terms( $product->get_id(), 'product_cat' )[0];
				$value    = $this->wgmt_feeds_get_google_taxonomy_value( $exp_term, $category );
				break;
			case 'static':
				$value = str_replace( '_', ' ', $exp_term );
				break;
			case 'product':
			default:
				$value = $product->$exp_term();
				break;
		}

		return $value;
	}

	/**
	 * Apply formats to value.
	 *
	 * @param $value
	 * @param $format
	 *
	 * @return string
	 */
	private function wgmt_feeds_format_value( $value, $format ) {
		if ( $format == 'money' ) {
			$value = number_format( $value, 2, '.', '' );
		}

		return $value;
	}

	/**
	 * Plugin values functions
	 *************************/

	/**
	 * Get the Google Taxonomy matched category.
	 *
	 * @param $exp
	 * @param WP_Term $category
	 *
	 * @return mixed|string
	 */
	private function wgmt_feeds_get_google_taxonomy_value( $exp, WP_Term $category ) {
		$result = "";

		$store_values = WGMT()->wgmt_get_option( 'store_1' );
		$taxonomies   = array_key_exists( 'taxonomies', $store_values ) ? $store_values['taxonomies'] : array();
		if ( ! empty( $taxonomies ) ) {
			$taxonomy = array_key_exists( $exp, $taxonomies ) ? $taxonomies[ $exp ] : array();
			if ( ! empty( $taxonomy ) ) {
				$result = array_key_exists( $category->slug, $taxonomy['mappings'] ) ? explode( ' - ', $taxonomy['mappings'][ $category->slug ] )[0] : "";
			}
		}

		return $result;
	}

	/**
	 * Get the product featured image url.
	 *
	 * @param $args
	 *
	 * @return false|string
	 */
	private function wgmt_feeds_get_self_value_image_featured( $args ) {
		/** @var \WC_Product $product */
		$product = $args['product'];

		return wp_get_attachment_url( $product->get_image_id() );
	}

	/**
	 * Get the product stock availability.
	 *
	 * @param $args
	 *
	 * @return string
	 */
	private function wgmt_feeds_get_self_value_availability( $args ) {
		/** @var \WC_Product $product */
		$product = $args['product'];

		if ( $product->is_in_stock() ) {
			return 'in stock';
		}

		return 'out of stock';
	}

	private function wgmt_feeds_get_self_is_bundle( $args ) {
		/** @var \WC_Product $product */
		$product = $args['product'];

		if ( $product->get_type() == 'grouped' ) {
			return 'yes';
		}

		return 'no';
	}

	private function wgmt_feeds_get_self_value_condition() {
		return 'New';
	}

	/**
	 * Get all the available values.
	 *
	 * @return array
	 */
	public static function wgmt_get_feeds_available_values() {
		return array(
			'Basic Product Info'    => array(
				'product_get_id'                => 'ID',
				'product_get_sku'               => 'SKU',
				'product_get_title'             => 'Title',
				'product_get_name'              => 'Name',
				'product_get_description'       => 'Description',
				'product_get_short_description' => 'Short Description',
				'product_get_permalink'         => 'Link',
				'self_image_featured'           => 'Featured Image',
				'term_product_cat'              => 'Category',
			),
			'Stock'                 => array(
				'self_availability'       => 'Availability',
				'product_get_price'       => 'Price',
				'self_condition'          => 'Condition',
				'product_get_sale_price'  => 'Sale Price',
				'product_get_total_sales' => 'Total Sales',
				'self_is_bundle'          => 'Bundle',
			),
			'Shipping'              => array(
				'product_get_weight' => 'Weight',
				'product_get_length' => 'Length',
				'product_get_width'  => 'Width',
				'product_get_height' => 'Height',
			),
			'Product Attributes'    => self::get_products_attributes(),
			'Product Custom Fields' => self::get_products_custom_meta(),
			'Static'                => array(
				'static_yes' => 'Yes',
				'static_no'  => 'No',
				'string_static' => 'Static Text',
			),
		);
	}

	private static function get_products_attributes() {
		$attributes = array();

		$wc_attributes_ids = wc_get_attribute_taxonomy_ids();
		foreach ( $wc_attributes_ids as $key => $id ) {
			$attribute                                = wc_get_attribute( $id );
			$attributes[ 'attr_' . $attribute->slug ] = $attribute->name;
		}

		return $attributes;
	}

	private static function get_products_custom_meta() {
		global $wpdb;

		$meta  = array();
		$table = $wpdb->prefix . 'postmeta';

		$rows = $wpdb->get_results( "SELECT DISTINCT meta.meta_key FROM $table AS meta WHERE meta.meta_key LIKE '%gtin%' OR meta.meta_key LIKE '%ean%' OR meta.meta_key LIKE '%upc%' OR meta.meta_key LIKE '%isbn%' OR meta.meta_key LIKE '%jan%' OR meta.meta_key LIKE '%itf14%' OR meta.meta_key LIKE '%mpn%' OR meta.meta_key LIKE '%woosea%' OR meta.meta_key LIKE '%wpm%'", ARRAY_A );

		foreach ( $rows as $row ) {
			$meta[ 'meta_' . $row['meta_key'] ] = $row['meta_key'];
		}

		// acf
		$table = $wpdb->prefix . 'posts';

		$rows = $wpdb->get_results( "SELECT DISTINCT post.post_excerpt FROM $table AS post WHERE post.post_type = 'acf-field'", ARRAY_A );
		foreach ( $rows as $row ) {
			$meta[ 'meta_' . $row['post_excerpt'] ] = 'acf_' . $row['post_excerpt'];
		}

		return $meta;
	}
}
