<?php

final class Merchant_Google_Shopping extends WGMT_Merchant {
	/**
	 * The feed to work with.
	 *
	 * @var array
	 */
	private $feed;

	public function __construct( $feed ) {
		$this->feed = $feed;
	}

	public function wgmt_generate_feed() {
		$format = $this->feed['config']['feeds_format'];
		$func   = 'generate_feed_' . $format;

		$feed_content = $this->$func();

		$path = wp_upload_dir();
		$path = $path['basedir'] . '/wgmt-feed';

		// make directory if not exist
		if ( ! file_exists( $path ) ) {
			wp_mkdir_p( $path );
		}

		/** Save feed */
		$name = $this->feed['id'];
		$file = trailingslashit( $path ) . "feed_{$name}.{$format}";
		file_put_contents( $file, $feed_content );
	}

	private function get_item_data( WC_Product $product ) {
		$main_condition = $this->feed['config']['feeds_then'];
		$tags           = $this->feed['config']['feeds_attr'];
		$item_data      = array();
		$preset         = self::wgmt_get_merchant_preset();

		foreach ( $tags as $tag ) {
			$key               = array_key_exists( 'custom', $tag ) ? strtolower( str_replace( ' ', '_', $tag['custom'] ) ) : $tag['name'];
			$value             = $this->wgmt_feeds_get_item_raw_value( $tag, $product );
			$passed_conditions = $this->wgmt_feeds_validate_tag( $product, $tag, $value, $main_condition );
			$formatted_value   = "";

			if ( ! empty( $value ) ) {
				if ( array_key_exists( 'custom', $tag ) ) {
					$formatted_value = $this->wgmt_feeds_get_formatted_value( $tag, $value );
				} else {
					foreach ( $preset as $field ) {
						if ( $tag['name'] == $field['attr'] ) {
							$formatted_value = $this->wgmt_feeds_get_formatted_value( $tag, $value, $field );
						}
					}
				}
			}

			$item_data[ $key ] = array(
				'valid'  => $passed_conditions,
				'value'  => $formatted_value,
				'verify' => ( array_key_exists( 'if', $tag ) && ! empty( $tag['if'] ) ) && ( array_key_exists( 'condition', $tag ) && ! empty( $tag['condition'] ) ),
			);
		}

		// Identifier exists
		if ( ! array_key_exists( 'gtin', $item_data ) && ! array_key_exists( 'MPN', $item_data ) ) {
			$item_data['identifier_exists'] = array(
				'valid'  => true,
				'value'  => 'no',
				'verify' => false,
			);
		}

		return $item_data;
	}

	private function generate_feed_xml() {
		$xml_charset     = get_option( 'blog_charset' );
		$xml_title       = 'Google Shopping XML Feed';
		$xml_link        = get_bloginfo( 'url' );
		$xml_description = 'Products Feed for the Google Shopping Merchant';

		$xml = "<?xml version=\"1.0\" encoding=\"{$xml_charset}\"?>";
		$xml .= "<rss xmlns:g=\"http://base.google.com/ns/1.0\" version=\"2.0\">";
		$xml .= "<channel>";

		$xml .= "<title>{$xml_title}</title>";
		$xml .= "<link>{$xml_link}</link>";
		$xml .= "<description>{$xml_description}</description>";

		$products = $this->wgmt_feeds_get_filtered_products( $this->feed );
		/** @var WC_Product $product */
		foreach ( $products as $product ) {
			$xml .= $this->wgmt_feeds_generate_xml_item( $product );
		}

		$xml .= "</channel>";
		$xml .= "</rss>";

		return $xml;
	}

	private function wgmt_feeds_generate_xml_item( WC_Product $product ) {
		$main_condition = $this->feed['config']['feeds_then'];
		$include        = ( $main_condition == '1' || $main_condition == '2' );
		$strict         = ( $main_condition == '1' || $main_condition == '-1' );

		$item_data = $this->get_item_data($product);

		$valid = true;
		foreach ( $item_data as $key => $datum ) {
			if ( $datum['verify'] ) {
				if ( $strict ) {
					$valid = $datum['valid'];

					if ( ! $valid ) {
						break;
					}
				} else {
					$valid = $datum['valid'];

					if ( $valid ) {
						break;
					}
				}
			}
		}

		/** Include or exclude? */
		if ( ! $include ) {
			$valid = ! $valid;
		}

		if ( $valid ) {
			$item = "<item>";

			foreach ( $item_data as $key => $datum ) {
				$item .= "<g:{$key}>";
				$item .= $datum['value'];
				$item .= "</g:{$key}>";
			}

			$item .= "</item>";

			return $item;
		}

		return "";
	}

	private function generate_feed_csv() {
		$csv = "";

		$products = $this->wgmt_feeds_get_filtered_products( $this->feed );
		$first    = true;
		/** @var WC_Product $product */
		foreach ( $products as $product ) {
			$csv   .= $this->wgmt_feeds_generate_csv_item( $product, $first );
			$first = false;
		}

		return $csv;
	}

	private function wgmt_feeds_generate_csv_item( WC_Product $product, $headers = false ) {
		$main_condition = $this->feed['config']['feeds_then'];
		$include        = ( $main_condition == '1' || $main_condition == '2' );
		$strict         = ( $main_condition == '1' || $main_condition == '-1' );

		$item_data = $this->get_item_data($product);

		$valid = true;
		foreach ( $item_data as $key => $datum ) {
			if ( $datum['verify'] ) {
				if ( $strict ) {
					$valid = $datum['valid'];

					if ( ! $valid ) {
						break;
					}
				} else {
					$valid = $datum['valid'];

					if ( $valid ) {
						break;
					}
				}
			}
		}

		/** Include or exclude? */
		if ( ! $include ) {
			$valid = ! $valid;
		}

		if ( $valid ) {
			$line = "";

			if ( $headers ) {
				foreach ( $item_data as $key => $datum ) {
					$line .= $key;
					$line .= ",";
				}
				$line .= "\n";
			}

			foreach ( $item_data as $key => $datum ) {
				$line .= "\"";
				$line .= $datum['value'];
				$line .= "\"";
				$line .= ',';
			}

			$line .= "\n";

			return $line;
		}

		return "";
	}

	public static function wgmt_get_merchant_format() {
		return 'xml';
	}

	public static function wgmt_get_merchant_attributes() {
		return array(
			'Basic Information'          => array(
				'id'                      => 'Product Id [id]',
				'title'                   => 'Product Title [title]',
				'description'             => 'Product Description [description]',
				'link'                    => 'Product URL [link]',
				'product_type'            => 'Product Category [product_type]',
				'google_product_category' => 'Google Taxonomy [google_product_category]',
				'image_link'              => 'Main Image [image_link]',
				'condition'               => 'Condition [condition]',
			),
			'Availability & Price'       => array(
				'availability' => 'Stock Status [availability]',
				'price'        => 'Regular Price [price]',
			),
			'Unique Product Identifiers' => array(
				'gtin' => 'GTIN [gtin]',
				'mpn'  => 'MPN [mpn]',
			),
			'Product Attributes'         => array(
				'item_group_id' => 'Item Group Id [item_group_id]',
				'brand'         => 'Manufacturer [brand]',
				'color'         => 'Color [color]',
				'gender'        => 'Gender [gender]',
				'age_group'     => 'Age Group [age_group]',
				'material'      => 'Material [material]',
				'pattern'       => 'Pattern [pattern]',
				'size'          => 'Size of the item [size]',
			),
			'Product Combinations'       => array(
				'multipack' => 'Multipack [multipack]',
				'is_bundle' => 'Is Bundle [is_bundle]',
			),
			'Adult Products'             => array(
				'adult' => 'Adult [adult]',
			),
		);
	}

	public static function wgmt_get_merchant_preset() {
		return array(
			array(
				'attr'           => 'id',
				'taken'          => 'product_get_id',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => false,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'title',
				'taken'          => 'product_get_title',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => true,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'description',
				'taken'          => 'product_get_description',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => true,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'link',
				'taken'          => 'product_get_permalink',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => true,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'image_link',
				'taken'          => 'self_image_featured',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => true,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'availability',
				'taken'          => 'self_availability',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => false,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'price',
				'taken'          => 'product_get_price',
				'dynamic_suffix' => get_woocommerce_currency(),
				'format'         => 'money',
				'cdata'          => false,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'google_product_category',
				'taken'          => '',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => true,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'brand',
				'taken'          => 'attr_pa_brand',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => true,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'gtin',
				'taken'          => 'attr_pa_gtin',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => true,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'MPN',
				'taken'          => 'product_get_sku',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => true,
				'mandatory'      => false,
			),
			array(
				'attr'           => 'condition',
				'taken'          => 'self_condition',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => true,
				'mandatory'      => true,
			),
			array(
				'attr'           => 'product_type',
				'taken'          => 'term_product_cat',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => true,
				'mandatory'      => false,
			),
			array(
				'attr'           => 'item_group_id',
				'taken'          => 'product_get_id',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => false,
				'mandatory'      => false,
			),
			array(
				'attr'           => 'color',
				'taken'          => 'attr_pa_color',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => false,
				'mandatory'      => false,
			),
			array(
				'attr'           => 'gender',
				'taken'          => 'attr_pa_gender',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => false,
				'mandatory'      => false,
			),
			array(
				'attr'           => 'material',
				'taken'          => 'attr_pa_material',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => false,
				'mandatory'      => false,
			),
			array(
				'attr'           => 'pattern',
				'taken'          => 'attr_pa_pattern',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => false,
				'mandatory'      => false,
			),
			array(
				'attr'           => 'is_bundle',
				'taken'          => 'self_is_bundle',
				'format'         => '',
				'dynamic_suffix' => '',
				'cdata'          => false,
				'mandatory'      => false,
			),
		);
	}
}
